# Copyright 2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""GPT model"""
import math
import numpy as np
import mindspore.nn as nn
from mindspore.common.tensor import Tensor
from mindspore.common.parameter import Parameter
import mindspore.common.dtype as mstype
from mindspore.common.initializer import initializer, Normal, Uniform, TruncatedNormal
from mindspore.ops import operations as P
from mindspore.ops import functional as F
import numpy as np
from mindspore.nn.transformer.transformer import VocabEmbedding
import copy
from mindspore.nn.transformer.layers import _Dropout

class EmbeddingLayer(nn.Cell):
    r"""Embedding layer of the PanGUAlpha Model"""
    def __init__(self, config):
        super(EmbeddingLayer, self).__init__()
        # Only for the pipeline mode, the embedding needs to be row sliced.
        self.word_embedding = VocabEmbedding(vocab_size=config.vocab_size,
                                             embedding_size=config.hidden_size,
                                             param_init=initializer("normal", [config.vocab_size, config.hidden_size],
                                                                    ),  # dtype=mstype.float32
                                             parallel_config=config.parallel_config.embedding_dp_mp_config)
        copied_parallel_config = copy.deepcopy(config.parallel_config)
        copied_parallel_config.vocab_emb_dp = True
        self.position_embedding = VocabEmbedding(vocab_size=config.seq_length,
                                                 embedding_size=config.hidden_size,
                                                 param_init=initializer("normal",
                                                                        [config.seq_length, config.hidden_size],
                                                                        ), # dtype=mstype.float32
                                                 parallel_config=copied_parallel_config.embedding_dp_mp_config)
        self.add = P.Add().shard(
            ((config.parallel_config.data_parallel, 1, 1), (config.parallel_config.data_parallel, 1, 1)))
        self.dropout = _Dropout(1 - config.dropout_rate)
        self.dropout.shard(((config.parallel_config.data_parallel, 1, 1),))
        self.is_first_iteration = True
        self.use_past = config.use_past
        self.batch_size = config.batch_size

    def construct(self, input_ids, input_position, init_reset, batch_valid_length):
        word_embedding, word_table = self.word_embedding(input_ids)
        if self.use_past and not self.is_first_iteration:
            _, seq_length = F.shape(input_ids)
            input_position = batch_valid_length.view(self.batch_size, seq_length)
        position_embedding, _ = self.position_embedding(input_position)
        embed = self.add(word_embedding, position_embedding)
        embed = self.dropout(embed)
        return embed, word_table

    def get_word_embedding_weight(self):
        return self.word_embedding.embedding_table

class LayerNormScale(nn.Cell):
    def __init__(self, normalized_shape, dp=4, eps=1e-5, scale=1e-3):
        super(LayerNorm, self).__init__()
        self.gamma = Parameter(initializer('ones', normalized_shape), name="gamma")
        self.beta = Parameter(initializer('zeros', normalized_shape), name="beta")
        self.mean = P.ReduceMean(keep_dims=True).shard(((dp, 1, 1),))
        self.square = P.Square().shard(((dp, 1, 1),))
        self.sqrt = P.Sqrt().shard(((dp, 1, 1),))
        self.sub1 = P.Sub().shard(((dp, 1, 1), (dp, 1, 1)))
        self.sub2 = P.Sub().shard(((dp, 1, 1), (dp, 1, 1)))
        self.add = P.TensorAdd().shard(((dp, 1, 1), ()))
        self.eps = eps
        self.mul = P.Mul().shard(((dp, 1, 1), (1,)))
        self.add2 = P.TensorAdd().shard(((dp, 1, 1), (1,)))
        self.real_div = P.RealDiv().shard(((dp, 1, 1), (dp, 1, 1)))
        self.scale_div = P.RealDiv().shard(((dp, 1, 1), ()))
        self.scale_mul = P.Mul().shard(((dp, 1, 1), ()))
        self.scale = scale
        self.scale_eps = self.eps / (self.scale * self.scale)

    def construct(self, x):
        mean = self.mean(x, -1)
        diff = self.sub1(x, mean)
        scale_diff = self.scale_div(diff, self.scale)
        scale_variance = self.mean(self.square(scale_diff), -1)
        scale_variance_eps = self.sqrt(self.add(scale_variance, self.scale_eps))
        variance_eps = self.scale_mul(scale_variance_eps, self.scale)
        output = self.real_div(diff, variance_eps)
        rescaled_output = self.add2(self.mul(output, self.gamma), self.beta)
        return rescaled_output


class LayerNorm(nn.Cell):
    def __init__(self, normalized_shape, dp=4, eps=1e-5, scale=1e-3):
        super(LayerNorm, self).__init__()
        self.gamma = Parameter(initializer('ones', normalized_shape), name="gamma")
        self.beta = Parameter(initializer('zeros', normalized_shape), name="beta")
        self.mean = P.ReduceMean(keep_dims=True).shard(((dp, 1, 1),))
        self.square = P.Square().shard(((dp, 1, 1),))
        self.sqrt = P.Sqrt().shard(((dp, 1, 1),))
        self.sub1 = P.Sub().shard(((dp, 1, 1), (dp, 1, 1)))
        self.sub2 = P.Sub().shard(((dp, 1, 1), (dp, 1, 1)))
        self.add = P.TensorAdd().shard(((dp, 1, 1), ()))
        self.eps = eps
        self.mul = P.Mul().shard(((dp, 1, 1), (1,)))
        self.add2 = P.TensorAdd().shard(((dp, 1, 1), (1,)))
        self.real_div = P.RealDiv().shard(((dp, 1, 1), (dp, 1, 1)))
        self.scale_div = P.RealDiv().shard(((dp, 1, 1), ()))
        self.scale_mul = P.Mul().shard(((dp, 1, 1), ()))
        self.scale = scale

    def construct(self, x):
        mean = self.mean(x, -1)
        diff = self.sub1(x, mean)
        variance = self.mean(self.square(diff), -1)
        variance_eps = self.sqrt(self.add(variance, self.eps))
        output = self.real_div(diff, variance_eps)
        output = self.add2(self.mul(output, self.gamma), self.beta)
        return output


class Mapping(nn.Cell):
    """
    A mapping function with a 3d input
    Args:
        input_size: the size of the last dimension of the input tensor
        output_size: the desired size of the last dimension of the output tensor
        dtype: the compute datatype
        scale: the scale factor for initialization
    Inputs:
        x: the 3d input
    Returns:
        output: Tensor, a 3d tensor after projection
    """

    # 优化：matmul，dtype, mapping_output
    def __init__(self, config, input_size, output_size, scale=1.0):
        super(Mapping, self).__init__()
        self.output_size = output_size
        self.input_size = input_size
        self.weight = Parameter(initializer(Normal(sigma=0.02 * scale),
                                            [input_size, output_size]),
                                name="mapping_weight")
        self.bias = Parameter(initializer("zeros", [
            output_size,
        ]),
                              name="mapping_bias",
                              parallel_optimizer=False)
        self.dtype = config.compute_dtype
        self.cast = P.Cast()
        self.add = P.TensorAdd().shard(((config.dp, 1), (1,)))
        self.matmul = P.MatMul().shard(
            ((config.dp, config.mp), (config.mp, 1)))

    def construct(self, x):
        out_shape = P.Shape()(x)[:-1] + (self.output_size,)
        x = P.Reshape()(x, (-1, self.input_size))
        weight = self.cast(self.weight, self.dtype)
        x = self.matmul(x, weight)
        x = self.add(x, self.cast(self.bias, self.dtype))
        output = P.Reshape()(x, out_shape)
        return output


class Mapping_output(nn.Cell):
    def __init__(self, config, input_size, output_size, scale=1.0):
        super(Mapping_output, self).__init__()
        self.output_size = output_size
        self.input_size = input_size
        self.weight = Parameter(initializer(Normal(sigma=0.02 * scale),
                                            [input_size, output_size]),
                                name="mapping_weight")
        self.bias = Parameter(initializer("zeros", [
            output_size,
        ]),
                              name="mapping_bias")
        self.dtype = config.compute_dtype
        self.cast = P.Cast()
        self.add = P.TensorAdd().shard(((config.dp, config.mp), (config.mp,)))
        self.matmul = P.MatMul().shard(((config.dp, 1), (1, config.mp)))

    def construct(self, x):
        out_shape = P.Shape()(x)[:-1] + (self.output_size,)
        x = P.Reshape()(x, (-1, self.input_size))
        weight = self.cast(self.weight, self.dtype)
        x = self.matmul(x, weight)
        x = self.add(x, self.cast(self.bias, self.dtype))
        output = P.Reshape()(x, out_shape)
        return output


class Output(nn.Cell):
    """
    The output mapping module for each layer
    Args:
        config(GPTConfig): the config of network
        scale: scale factor for initialization
    Inputs:
        x: output of the self-attention module
    Returns:
        output: Tensor, the output of this layer after mapping
    """

    def __init__(self, config, scale=1.0):
        super(Output, self).__init__()
        input_size = config.embedding_size
        output_size = config.embedding_size * config.expand_ratio
        self.mapping = Mapping_output(config, input_size, output_size)
        self.projection = Mapping(config, output_size, input_size, scale)
        self.activation = nn.GELU()
        self.activation.gelu.shard(((config.dp, 1, config.mp),))
        self.dropout = nn.Dropout(1 - config.dropout_rate)
        self.dropout.dropout.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_gen_mask.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_do_mask.shard(((config.dp, 1, 1),))

    def construct(self, x):
        hidden = self.activation(self.mapping(x))
        output = self.projection(hidden)
        output = self.dropout(output)
        return output


class AttentionMask(nn.Cell):
    """
    Get the attention matrix for self-attention module
    Args:
        config(GPTConfig): the config of network
    Inputs:
        input_mask: the mask indicating whether each position is a valid input
    Returns:
        attention_mask: the attention mask matrix with shape (batch_size, 1, seq_length, seq_length)
    """

    def __init__(self, config):
        super(AttentionMask, self).__init__()
        self.reshape = P.Reshape()
        self.mul = P.BatchMatMul().shard(
            ((config.dp, 1, 1), (config.dp, 1, 1)))  # yzz: use 64, 1, 1?
        self.expand_dim = P.ExpandDims().shard(((1, 1),))
        ones = np.ones(shape=(config.seq_length, config.seq_length))
        self.lower_triangle_mask = Tensor(np.tril(ones), mstype.float32)
        self.multiply = P.Mul().shard(((config.dp, 1, 1), (1, 1, 1)))

    def construct(self, input_mask):
        input_shape = P.Shape()(input_mask)
        shape_right = (input_shape[0], 1, input_shape[1])
        shape_left = input_shape + (1,)
        mask_left = self.reshape(input_mask, shape_left)
        mask_right = self.reshape(input_mask, shape_right)
        attention_mask = self.mul(mask_left, mask_right)
        lower_traiangle = self.expand_dim(self.lower_triangle_mask, 0)
        attention_mask = self.multiply(
            attention_mask, lower_traiangle)  # bs seq_length seq_length
        return attention_mask


class EmbeddingLookup(nn.Cell):
    """
    The embedding lookup table for vocabulary
    Args:
        config(GPTConfig): the config of network
    Inputs:
        input_ids: the tokenized inputs with datatype int32
    Returns:
        output: Tensor, the embedding vector for the input with shape (batch_size, seq_length, embedding_size)
        self.embedding_table: Tensor, the embedding table for the vocabulary
    """

    def __init__(self, config):
        super(EmbeddingLookup, self).__init__()
        self.vocab_size = config.vocab_size
        self.embedding_size = config.embedding_size
        try:
            e_table = np.load('{}/word_embedding.npy'.format(config.load_ckpt_local_path))
            e_table = Tensor(e_table, mstype.float32)
            print("###### loading Word embedding Table successed! ######")
        except:
            e_table = Tensor(np.ones([self.vocab_size, self.embedding_size], dtype=np.float32))
        self.embedding_table = Parameter(e_table,
                                         name="embedding_table")
        if config.word_emb_dp:
            self.gather = P.GatherV2().shard(((1, 1), (config.dp, 1)))
        else:
            self.gather = P.GatherV2().shard(((1, config.mp), (config.dp, 1)))
            self.gather.add_prim_attr("reverse_stra", True)
            print("Using the reverse_stra columns slice", flush=True)
        self.shape = (-1, config.seq_length, config.embedding_size)

    def construct(self, input_ids):
        output = self.gather(self.embedding_table, input_ids, 0)
        return output, self.embedding_table


class Attention(nn.Cell):
    """
    Self-Attention module for each layer

    Args:
        config(GPTConfig): the config of network
        scale: scale factor for initialization
        layer_idx: current layer index
    """

    def __init__(self, config, scale=1.0, layer_idx=None):
        super(Attention, self).__init__()
        self.get_attention_mask = AttentionMask(config)
        self.projection = Mapping(config, config.embedding_size,
                                  config.embedding_size, scale)
        self.transpose = P.Transpose().shard(((config.dp, 1, config.mp, 1),))
        self.merger_head_transpose = P.Transpose().shard(
            ((config.dp, config.mp, 1, 1),))
        self.reshape = P.Reshape()
        self.n_head = config.num_heads
        self.size_per_head = config.embedding_size // self.n_head
        self.concat_k = P.Concat(axis=3)
        self.concat_v = P.Concat(axis=2)
        self.multiply_data = Tensor([
            -10000.0,
        ], dtype=mstype.float32)
        self.batch_matmul = P.BatchMatMul().shard(
            ((config.dp, config.mp, 1, 1), (config.dp, config.mp, 1, 1)))
        self.scale = scale
        self.real_div = P.RealDiv().shard(((config.dp, config.mp, 1, 1), ()))
        self.sub = P.Sub().shard(
            ((1,), (config.dp, 1, 1, 1))).add_prim_attr("_side_effect", True)
        self.mul = P.Mul().shard(
            ((config.dp, 1, 1, 1), (1,))).add_prim_attr("_side_effect", True)
        self.add = P.TensorAdd().shard(
            ((config.dp, 1, 1, 1), (config.dp, config.mp, 1, 1)))
        if self.scale:
            self.scale_factor = Tensor(math.sqrt(self.size_per_head))
        if layer_idx is not None:
            self.coeff = math.sqrt(layer_idx * math.sqrt(self.size_per_head))
            self.coeff = Tensor(self.coeff)
        self.use_past = config.use_past
        self.dropout = nn.Dropout(1 - config.dropout_rate)
        self.dropout.dropout.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_gen_mask.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_do_mask.shard(((config.dp, 1, 1),))
        self.prob_dropout = nn.Dropout(1 - config.dropout_rate)
        self.prob_dropout.dropout.shard(((config.dp, config.mp, 1, 1),))
        #         self.prob_dropout.dropout_gen_mask.shard(
        #             ((config.dp, config.mp, 1, 1),))
        #         self.prob_dropout.dropout_do_mask.shard(
        #             ((config.dp, config.mp, 1, 1),))
        self.softmax = nn.Softmax()
        self.softmax.softmax.shard(((config.dp, config.mp, 1),))
        self.expand_dims = P.ExpandDims().shard(((config.dp, 1, 1),))

        self.dense1 = nn.Dense(config.embedding_size,
                               config.embedding_size).to_float(
            config.compute_dtype)
        self.dense1.matmul.shard(((config.dp, 1), (config.mp, 1)))
        self.dense1.bias_add.shard(((config.dp, config.mp), (config.mp,)))
        self.dense2 = nn.Dense(config.embedding_size,
                               config.embedding_size).to_float(
            config.compute_dtype)
        self.dense2.matmul.shard(((config.dp, 1), (config.mp, 1)))
        self.dense2.bias_add.shard(((config.dp, config.mp), (config.mp,)))
        self.dense3 = nn.Dense(config.embedding_size,
                               config.embedding_size).to_float(
            config.compute_dtype)
        self.dense3.matmul.shard(((config.dp, 1), (config.mp, 1)))
        self.dense3.bias_add.shard(((config.dp, config.mp), (config.mp,)))

        self.use_past = config.use_past
        if self.use_past:
            self.pad_value = 0
            seq_range = np.arange(config.seq_length).reshape(1, 1, -1)
            self.range = Tensor(seq_range, mstype.int32)
            self.seq_length = config.seq_length
            self.slice = P.StridedSlice().shard(((1, 1, 1, 1),))
            self.not_equal = P.NotEqual().shard(((1, 1, 1, 1), ()))
            self.reducesum = P.ReduceSum().shard(((1, 1, 1, 1),))
            self.expand_dims = P.ExpandDims().shard(((1, 1, 1),))
            self.tensor_le = P.LessEqual().shard(((1, 1, 1, 1), (1, 1, 1, 1)))
            self.add = P.TensorAdd().shard(((1, 1, 1, 1), (1, 1, 1, 1)))
            self.equal = P.Equal().shard(((1, 1, 1), (1, 1, 1)))
            self.sub1 = P.Sub().shard(((1,), ()))
            self.tile = P.Tile().shard(((1, 1, 1, 1),))
            self.tile1 = P.Tile().shard(((1, 1, 1),))
            self.is_first_iteration = True
            self.less = P.Less().shard(((1, 1, 1), (1, 1, 1)))
            self.mul1 = P.Mul().shard(((1, 1, 1, 1), (1, 1, 1, 1)))
            self.dtype = config.compute_dtype

    def construct(self, x, attention_mask, key_past=None, value_past=None, batch_valid_length=None):
        """
        self-attention

        Inputs:
            x: output of previous layer
            attention_mask: the attention mask matrix with shape (batch_size,
            1, seq_length, seq_length)
            key_past: the previous key of the self-attention module
            value_past: the previous value of the self-attention module
            batch_valid_length: the original valid sentence length of the first graph for incremental inference

        Returns:
            output: Tensor, the output logit of this layer
            layer_present: Tensor, the feature map of current layer
        """

        original_shape = F.shape(x)
        x = F.reshape(x, (-1, original_shape[-1]))
        query = self.dense1(x)
        key = self.dense2(x)
        value = self.dense3(x)
        query = self.transpose(
            F.reshape(
                query,
                (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 1, 3))
        key = self.transpose(
            F.reshape(
                key, (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 3, 1))
        value = self.transpose(
            F.reshape(
                value,
                (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 1, 3))

        key_present = key
        value_present = value
        range_vector = None
        if self.use_past:
            range_vector = self.tile1(self.range, (original_shape[0], 1, 1))
            if self.is_first_iteration:
                valid_length_vector = F.cast(self.less(range_vector, batch_valid_length), mstype.float16)
                key_present = self.mul1(key, self.expand_dims(valid_length_vector, 2))
                value_present = self.mul1(value, self.expand_dims(valid_length_vector, 3))
            else:
                valid_length = self.reducesum(F.cast(self.not_equal(
                    self.slice(key_past, (0, 0, 0, 0), (original_shape[0], 1, 1, self.seq_length), (1, 1, 1, 1)),
                    self.pad_value), mstype.float32), (1, 2, 3))
                valid_length = F.reshape(valid_length, (-1, 1, 1))
                valid_length_vector = F.cast(self.equal(valid_length, range_vector), self.dtype)
                current_key = self.mul1(self.tile(key, (1, 1, 1, self.seq_length)),
                                        self.expand_dims(valid_length_vector, 2))
                current_value = self.mul1(self.tile(value, (1, 1, self.seq_length, 1)),
                                          self.expand_dims(valid_length_vector, 3))
                key = self.add(key_past, current_key)
                value = self.add(value_past, current_value)
                key_present = key
                value_present = value
        layer_present = (key_present, value_present)
        attention = self._attn(query, key, value, attention_mask, range_vector)
        attention_merge = self.merge_heads(attention)
        output = self.projection(attention_merge)
        output = self.dropout(output)
        return output, layer_present

    def split_heads(self, x, transpose):
        """
        split 3d tensor to 4d and switch certain axes
        Inputs:
            x: input tensor
            transpose: tuple, the transpose sequence
        Returns:
            x_transpose: the 4d output
        """
        x_size = P.Shape()(x)
        new_x_shape = x_size[:-1] + (self.n_head, self.size_per_head)
        x = self.reshape(x, new_x_shape)
        x_transpose = self.transpose(x, transpose)
        return x_transpose

    def merge_heads(self, x):
        """
        convert a 4d input to a 3d output

        Inputs:
            x: input tensor

        Returns:
            x_merge: the 3d output
        """
        x = self.merger_head_transpose(
            x, (0, 2, 1, 3))  # bs, seq_length, head, size_per_head
        x_shape = P.Shape()(x)
        new_shape = x_shape[:-2] + (x_shape[-2] * x_shape[-1],)
        x_merge = self.reshape(x, new_shape)
        return x_merge

    def _attn(self, query, key, value, attention_mask, range_vector=None):
        """
        Get the weighted score along the seq_length

        Inputs:
            query: the query matrix
            key: the key matrix
            value: the value matrix
            attention_mask: the attention mask matrix with shape (batch_size,
            1, seq_length, seq_length)
            range_vector: a range vector from 0 to seq_length with shape (batch_size, 1, seq_length)
        Returns:
            weighted_values: Tensor, the weighted sum scores
        """
        if not self.scale:
            query = query / F.cast(self.coeff, F.dtype(query))
            key = key / F.cast(self.coeff, F.dtype(key))

        score = self.batch_matmul(query, key)
        if self.scale:
            score = self.real_div(
                score,
                P.Cast()(self.scale_factor, P.DType()(score)))

        ori_dtype = P.DType()(score)
        score = P.Cast()(score, mstype.float32)

        if self.use_past and not self.is_first_iteration:
            current_index = self.reducesum(F.cast(
                self.not_equal(self.slice(key, (0, 0, 0, 0), (F.shape(query)[0], 1, 1, self.seq_length), (1, 1, 1, 1)),
                               self.pad_value), mstype.float32), (1, 2, 3))
            index = self.sub1(F.cast(current_index, mstype.int32), 1)
            index = F.reshape(index, (-1, 1, 1))
            index = self.expand_dims(index, 2)
            attention_mask = F.cast(self.tensor_le(self.expand_dims(range_vector, 2), index), mstype.int32)

            # attention_mask = F.cast(self.tensor_le(range_vector, index), mstype.int32)
            # attentoin_mask = self.expand_dims(attention_mask, 2)

        multiplu_out = self.sub(
            P.Cast()(F.tuple_to_array((1.0,)), P.DType()(score)),
            P.Cast()(attention_mask, P.DType()(score)))

        adder = self.mul(multiplu_out, self.multiply_data)
        attention_scores = self.add(adder, score)

        # attention_scores = P.Cast()(attention_scores, ori_dtype)
        shape = F.shape(attention_scores)
        attention_probs = self.softmax(
            F.reshape(attention_scores,
                      (shape[0], -1, shape[-1])))  # yzz modify
        attention_probs = P.Cast()(attention_probs, ori_dtype)
        attention_probs = F.reshape(attention_probs, shape)

        attention_probs = self.prob_dropout(attention_probs)
        weighted_values = self.batch_matmul(attention_probs, value)
        return weighted_values


class Block(nn.Cell):
    """
    The basic block of GPT network
    Args:
        config(GPTConfig): the config of network
        layer_idx: current layer index
    Inputs:
        x: the output of previous layer(input_ids for the first layer)
        attention_mask: the attention mask matrix with shape (batch_size, 1, seq_length, seq_length)
        layer_past: the previous feature map
    Returns:
        output: Tensor, the output logit of this layer
        layer_present: Tensor, the feature map of current layer
    """

    def __init__(self, config, layer_idx):
        super(Block, self).__init__()
        scale = 1 / math.sqrt(2.0 * config.num_layers)
        # scale = 1.0
        if config.self_layernorm:
            self.layernorm1 = LayerNorm((config.embedding_size,), config.dp).to_float(mstype.float32)
            self.layernorm2 = LayerNorm((config.embedding_size,), config.dp).to_float(mstype.float32)
        else:
            self.layernorm1 = nn.LayerNorm((config.embedding_size,)).to_float(mstype.float32)
            self.layernorm1.layer_norm.shard(((config.dp, 1, 1), (1,), (1,)))
            self.layernorm2 = nn.LayerNorm((config.embedding_size,)).to_float(mstype.float32)
            self.layernorm2.layer_norm.shard(((config.dp, 1, 1), (1,), (1,)))

        self.layernorm1.gamma.parallel_optimizer = False
        self.layernorm1.beta.parallel_optimizer = False
        self.attention = Attention(config, scale, layer_idx)
        self.layernorm2.gamma.parallel_optimizer = False
        self.layernorm2.beta.parallel_optimizer = False
        self.output = Output(config, scale)
        self.post_layernorm_residual = config.post_layernorm_residual
        self.add = P.TensorAdd().shard(((config.dp, 1, 1), (config.dp, 1, 1)))
        self.last_add = P.TensorAdd().shard(
            ((config.dp, 1, 1), (config.dp, 1,
                                 1))).add_prim_attr("recompute", False)
        self.dtype = config.compute_dtype

    def construct(self, x, input_mask, key_past=None, value_past=None, batch_valid_length=None):
        input_x = self.layernorm1(x)
        input_x = F.cast(input_x, self.dtype)
        attention, layer_present = self.attention(input_x, input_mask,
                                                  key_past, value_past, batch_valid_length)
        if self.post_layernorm_residual:
            x = self.add(input_x, attention)
        else:
            x = self.add(x, attention)

        output_x = self.layernorm2(x)
        output_x = F.cast(output_x, self.dtype)
        mlp_logit = self.output(output_x)
        if self.post_layernorm_residual:
            output = self.last_add(output_x, mlp_logit)
        else:
            output = self.last_add(x, mlp_logit)
        return output, layer_present


class QueryLayerAttention(Attention):
    def construct(self, x, query_hidden_state, attention_mask, key_past=None, value_past=None, batch_valid_length=None):
        original_shape = F.shape(x)
        x = F.reshape(x, (-1, original_shape[-1]))
        query_hidden_state = F.reshape(query_hidden_state, (-1, original_shape[-1]))
        query = self.dense1(query_hidden_state)
        key = self.dense2(x)
        value = self.dense3(x)
        query = self.transpose(
            F.reshape(
                query,
                (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 1, 3))
        key = self.transpose(
            F.reshape(
                key, (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 3, 1))
        value = self.transpose(
            F.reshape(
                value,
                (-1, original_shape[1], self.n_head, self.size_per_head)),
            (0, 2, 1, 3))
        key_present = key
        value_present = value
        range_vector = None
        if self.use_past:
            range_vector = self.tile1(self.range, (original_shape[0], 1, 1))
            if self.is_first_iteration:
                valid_length_vector = F.cast(self.less(range_vector, batch_valid_length), mstype.float16)
                key_present = self.mul1(key, self.expand_dims(valid_length_vector, 2))
                value_present = self.mul1(value, self.expand_dims(valid_length_vector, 3))
            else:
                valid_length = self.reducesum(F.cast(self.not_equal(
                    self.slice(key_past, (0, 0, 0, 0), (original_shape[0], 1, 1, self.seq_length), (1, 1, 1, 1)),
                    self.pad_value), mstype.float32), (1, 2, 3))
                valid_length = F.reshape(valid_length, (-1, 1, 1))
                valid_length_vector = F.cast(self.equal(valid_length, range_vector), self.dtype)
                current_key = self.mul1(self.tile(key, (1, 1, 1, self.seq_length)),
                                        self.expand_dims(valid_length_vector, 2))
                current_value = self.mul1(self.tile(value, (1, 1, self.seq_length, 1)),
                                          self.expand_dims(valid_length_vector, 3))
                key = self.add(key_past, current_key)
                value = self.add(value_past, current_value)
                key_present = key
                value_present = value
        layer_present = (key_present, value_present)
        attention = self._attn(query, key, value, attention_mask, range_vector)
        attention_merge = self.merge_heads(attention)
        output = self.projection(attention_merge)
        output = self.dropout(output)
        return output, layer_present


class QueryLayer(nn.Cell):
    def __init__(self, config):
        super(QueryLayer, self).__init__()
        scale = 1 / math.sqrt(2.0 * config.num_layers)
        self.layernorm1 = LayerNorm((config.embedding_size,), config.dp).to_float(mstype.float32)
        self.layernorm2 = LayerNorm((config.embedding_size,), config.dp).to_float(mstype.float32)
        self.layernorm1.gamma.parallel_optimizer = False
        self.layernorm1.beta.parallel_optimizer = False
        self.attention = QueryLayerAttention(config, scale)
        self.layernorm2.gamma.parallel_optimizer = False
        self.layernorm2.beta.parallel_optimizer = False
        self.output = Output(config, scale)
        self.post_layernorm_residual = config.post_layernorm_residual
        self.add = P.TensorAdd().shard(((config.dp, 1, 1), (config.dp, 1, 1)))

        self.last_add = P.TensorAdd().shard(
            ((config.dp, 1, 1), (config.dp, 1,
                                 1))).add_prim_attr("recompute", False)
        self.dtype = config.compute_dtype

    def construct(self, x, query_hidden_state, input_mask, key_past=None, value_past=None, batch_valid_length=None):
        input_x = self.layernorm1(x)
        input_x = F.cast(input_x, self.dtype)
        attention, layer_present = self.attention(input_x,
                                                  query_hidden_state,
                                                  input_mask,
                                                  key_past, value_past, batch_valid_length)
        if self.post_layernorm_residual:
            x = self.add(input_x, attention)
        else:
            x = self.add(x, attention)

        output_x = self.layernorm2(x)
        output_x = F.cast(output_x, self.dtype)
        mlp_logit = self.output(output_x)
        if self.post_layernorm_residual:
            output = self.last_add(output_x, mlp_logit)
        else:
            output = self.last_add(x, mlp_logit)
        return output, layer_present


class PanguAlpha_Model(nn.Cell):
    """
    The backbone of GPT network
    Args:
        config(GPTConfig): the config of network
    Inputs:
        input_ids: the tokenized inputs with datatype int32
        input_mask: the mask indicating whether each position is a valid input
        layer_past: the previous feature map
    Returns:
        output_state: Tensor, the output logit of backbone
        present_layer: Tensor, the current feature map
        embedding_table: Tensor, the embedding table for the vocabulary
    """

    def __init__(self, config):
        super(PanguAlpha_Model, self).__init__()
        self.get_attention_mask = AttentionMask(config)
        self.word_embedding = EmbeddingLookup(config).set_comm_fusion(1)
        try:
            p_table = np.load('{}/position_embedding.npy'.format(config.load_ckpt_local_path))
            p_table = Tensor(p_table, mstype.float32)
            print("###### loading Word embedding Table successed! ######")
        except:
            p_table = Tensor(np.ones([config.seq_length, config.embedding_size], dtype=np.float32))
        self.position_embedding = nn.Embedding(
            config.seq_length,
            config.embedding_size,
            embedding_table=p_table).set_comm_fusion(1)
        self.word_embedding.embedding_table.parallel_optimizer = False
        self.position_embedding.embedding_table.parallel_optimizer = False
        self.position_embedding.gather.shard(((1, 1), (config.dp,)))
        self.position_embedding.expand.shard(((config.dp, 1),))
        # self.position_embedding.gather.shard(((config.mp, 1), (1,)))
        # self.position_embedding.expand.shard(((1, 1),))
        self.blocks = nn.CellList()
        fusion_group_num = 4
        fusion_group_size = config.num_layers // fusion_group_num
        fusion_group_size = max(fusion_group_size, 1)

        num_layers = config.num_layers
        if config.use_top_query_attention:
            num_layers -= 1
        self.num_layers = num_layers
        print("After setting the layer is:", num_layers, flush=True)

        for i in range(num_layers):
            per_block = Block(config, i + 1).set_comm_fusion(int(i / fusion_group_size) + 2)
            per_block.recompute()
            per_block.attention.dropout.dropout.recompute(False)
            per_block.attention.prob_dropout.dropout.recompute(False)
            per_block.output.dropout.dropout.recompute(False)
            per_block.attention.dropout.dropout.add_prim_attr("_side_effect", True)
            per_block.attention.prob_dropout.dropout.add_prim_attr("_side_effect", True)
            per_block.output.dropout.dropout.add_prim_attr("_side_effect", True)
            self.blocks.append(per_block)

        if config.self_layernorm:
            self.layernorm = LayerNorm((config.embedding_size,), config.dp).to_float(
                mstype.float32).set_comm_fusion(
                int((num_layers - 1) / fusion_group_size) + 2)
        else:
            self.layernorm = nn.LayerNorm((config.embedding_size,)).to_float(
                mstype.float32).set_comm_fusion(
                int((num_layers - 1) / fusion_group_size) + 2)
            self.layernorm.layer_norm.shard(((config.dp, 1, 1), (1,), (1,)))
        self.layernorm.gamma.parallel_optimizer = False
        self.layernorm.beta.parallel_optimizer = False
        self.use_past = config.use_past
        self.past = tuple([None] * config.num_layers)
        self.add = P.TensorAdd().shard(((config.dp, 1, 1), (config.dp, 1, 1)))
        self.expand_dims = P.ExpandDims().shard(((config.dp, 1, 1),))
        self.dtype = config.compute_dtype
        self.dropout = nn.Dropout(1 - config.dropout_rate)
        self.dropout.dropout.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_gen_mask.shard(((config.dp, 1, 1),))
        # self.dropout.dropout_do_mask.shard(((config.dp, 1, 1),))
        self.eod_reset = config.eod_reset
        if config.use_top_query_attention:
            print("Using query layer...", flush=True)
            try:
                query_table = np.load('{}/top_query_embedding.npy'.format(config.load_ckpt_local_path))
                query_table = Tensor(query_table, mstype.float32)
                print("###### loading Word embedding Table successed! ######")
            except:
                query_table = Tensor(np.ones([config.seq_length, config.embedding_size], dtype=np.float32))
            self.top_query_embedding = nn.Embedding(config.seq_length, config.embedding_size, \
                                                    embedding_table=query_table).set_comm_fusion(
                int((config.num_layers - 1) / fusion_group_num) + 2)
            self.top_query_embedding.embedding_table.parallel_optimizer = False
            self.top_query_embedding.gather.shard(((1, 1), (config.dp,)))
            self.top_query_embedding.expand.shard(((config.dp, 1),))
            self.top_query_layer = QueryLayer(config)
            if config.use_recompute:
                self.top_query_layer.recompute()
                self.top_query_layer.output.dropout.dropout.recompute(False)
                self.top_query_layer.attention.dropout.dropout.recompute(False)
                self.top_query_layer.attention.prob_dropout.dropout.recompute(False)
                self.top_query_layer.output.dropout.dropout.add_prim_attr("_side_effect", True)
                self.top_query_layer.attention.dropout.dropout.add_prim_attr("_side_effect", True)
                self.top_query_layer.attention.prob_dropout.dropout.add_prim_attr("_side_effect", True)
            #                 self.top_query_layer.output.dropout.dropout_gen_mask.recompute(False)
            #                 self.top_query_layer.attention.dropout.dropout_gen_mask.recompute(False)
            #                 self.top_query_layer.attention.prob_dropout.dropout_gen_mask.recompute(False)
            #                 self.top_query_layer.output.dropout.dropout_gen_mask.add_prim_attr("_side_effect", True)
            #                 self.top_query_layer.attention.dropout.dropout_gen_mask.add_prim_attr("_side_effect", True)
            #                 self.top_query_layer.attention.prob_dropout.dropout_gen_mask.add_prim_attr("_side_effect", True)

            self.top_query_layer.set_comm_fusion(int((config.num_layers - 1) / fusion_group_num) + 2)
        self.use_top_query_attention = config.use_top_query_attention

        if self.use_past:
            self.pad_value = 0
            self.size_per_head = int(config.embedding_size / config.num_heads)
            key_past = np.zeros(
                shape=(config.num_layers, config.batch_size, config.num_heads, self.size_per_head, config.seq_length))
            value_past = np.zeros(
                shape=(config.num_layers, config.batch_size, config.num_heads, config.seq_length, self.size_per_head))
            self.dtype = config.compute_dtype
            self.key_past = Parameter(Tensor(key_past, self.dtype), name="key_past")
            self.value_past = Parameter(Tensor(value_past, self.dtype), name="value_past")
            self.reducesum = P.ReduceSum().shard(((1, 1, 1, 1, 1),))
            self.not_equal = P.NotEqual().shard(((1, 1, 1, 1, 1), ()))
            self.slice = P.StridedSlice().shard(((1, 1, 1, 1, 1),))
            self.expand_dims1 = P.ExpandDims().shard(((1, 1, 1, 1),))
            self.update = P.ScatterUpdate().shard(((1, 1, 1, 1, 1), (1,), (1, 1, 1, 1, 1)))
            self.num_heads = config.num_heads
            self.seq_length = config.seq_length
            self.mul = P.Mul().shard(((1, 1, 1, 1, 1), (1,)))
            self.assign = P.Assign().shard(((1, 1, 1, 1, 1), (1, 1, 1, 1, 1)))
            self.is_first_iteration = True
            self.depend = P.Depend()

    def construct(self, input_ids, input_mask, init_reset=True, batch_valid_length=None, input_position=None,
                  attention_mask=None,
                  layer_past=None):
        """GPT model"""
        if not self.use_past:
            layer_past = self.past

        input_embedding, embedding_table = self.word_embedding(input_ids)
        if not self.eod_reset:
            batch_size, seq_length = F.shape(input_ids)
            input_position = F.tuple_to_array(F.make_range(seq_length))
            input_position = P.Tile()(input_position, (batch_size, 1))
            attention_mask = self.get_attention_mask(input_mask)

        if self.use_past and not self.is_first_iteration:
            index = self.reducesum(F.cast(self.not_equal(
                self.slice(self.key_past, (0, 0, 0, 0, 0), (1, F.shape(input_ids)[0], 1, 1, self.seq_length),
                           (1, 1, 1, 1, 1)), self.pad_value), mstype.float32), (0, 2, 3, 4))
            index = F.cast(index, mstype.int32)
            input_position = F.reshape(index, (-1, 1))
        position_embedding = self.position_embedding(input_position)
        hidden_states = self.add(input_embedding, position_embedding)
        hidden_states = self.dropout(hidden_states)
        hidden_states = P.Cast()(hidden_states, mstype.float16)
        attention_mask = self.expand_dims(attention_mask, 1)

        a = None
        b = None
        if self.use_past:
            a = self.assign(self.key_past, self.mul(self.key_past, F.cast(init_reset, self.dtype)))
            b = self.assign(self.value_past, self.mul(self.value_past, F.cast(init_reset, self.dtype)))
        present_layer = ()
        for i in range(self.num_layers):
            key_past = None
            value_past = None
            if self.use_past:
                key_past = self.slice(self.key_past, (i, 0, 0, 0, 0), (
                    i + 1, F.shape(input_ids)[0], self.num_heads, self.size_per_head, self.seq_length), (1, 1, 1, 1, 1))
                value_past = self.slice(self.value_past, (i, 0, 0, 0, 0), (
                    i + 1, F.shape(input_ids)[0], self.num_heads, self.seq_length, self.size_per_head), (1, 1, 1, 1, 1))
                key_past = F.reshape(key_past,
                                     (F.shape(input_ids)[0], self.num_heads, self.size_per_head, self.seq_length))
                value_past = F.reshape(value_past,
                                       (F.shape(input_ids)[0], self.num_heads, self.seq_length, self.size_per_head))
            hidden_states, present = self.blocks[i](hidden_states,
                                                    attention_mask, key_past, value_past, batch_valid_length)
            key_present, value_present = present

            if self.use_past:
                c = self.update(self.key_past, F.reshape(P.ScalarToTensor()(i, mstype.int32), (1,)),
                                self.expand_dims1(key_present, 0))
                d = self.update(self.value_past, F.reshape(P.ScalarToTensor()(i, mstype.int32), (1,)),
                                self.expand_dims1(value_present, 0))
                tmp_ac = self.depend(a, c)
                tmp_bd = self.depend(b, d)
        output_state = self.layernorm(hidden_states)
        output_state = F.cast(output_state, self.dtype)
        if self.use_top_query_attention:
            key_past = None
            value_past = None
            i = self.num_layers
            if self.use_past:
                key_past = self.slice(self.key_past, (i, 0, 0, 0, 0), (
                    i + 1, F.shape(input_ids)[0], self.num_heads, self.size_per_head, self.seq_length), (1, 1, 1, 1, 1))
                value_past = self.slice(self.value_past, (i, 0, 0, 0, 0), (
                    i + 1, F.shape(input_ids)[0], self.num_heads, self.seq_length, self.size_per_head), (1, 1, 1, 1, 1))
                key_past = F.reshape(key_past,
                                     (F.shape(input_ids)[0], self.num_heads, self.size_per_head, self.seq_length))
                value_past = F.reshape(value_past,
                                       (F.shape(input_ids)[0], self.num_heads, self.seq_length, self.size_per_head))
            # top_query_hidden_states = self.top_query_embedding(input_position_seq)
            top_query_hidden_states = self.top_query_embedding(input_position)
            output_state, present = self.top_query_layer(output_state, top_query_hidden_states,
                                                         attention_mask, key_past, value_past, batch_valid_length)
            key_present, value_present = present

            if self.use_past:
                c = self.update(self.key_past, F.reshape(P.ScalarToTensor()(i, mstype.int32), (1,)),
                                self.expand_dims1(key_present, 0))
                d = self.update(self.value_past, F.reshape(P.ScalarToTensor()(i, mstype.int32), (1,)),
                                self.expand_dims1(value_present, 0))
                tmp_ac = self.depend(a, c)
                tmp_bd = self.depend(b, d)
        return output_state, present_layer, embedding_table

class PanGuHead(nn.Cell):
    """
    Head for GPT to get the logits of each token in the vocab
    Args:
        config(GPTConfig): the config of network
    Inputs:
        state: the output of the backbone
        embedding_table: the embedding table of the vocabulary
    Returns:
        logits: Tensor, the logits of the corresponding inputs
    """

    def __init__(self, config):
        super(PanGuHead, self).__init__()
        if config.word_emb_dp:
            self.matmul = P.MatMul(transpose_b=True).shard(((config.dp, 1), (1, 1)))
        else:
            self.matmul = P.MatMul(transpose_b=True).shard(((config.dp, config.mp), (1, config.mp)))
        self.embedding_size = config.embedding_size
        self.log_softmax = P.LogSoftmax(axis=-1)
        self.dtype = config.compute_dtype
        self.cast = P.Cast()

    def construct(self, state, embedding_table):
        state = P.Reshape()(state, (-1, self.embedding_size))
        logits = self.matmul(state, self.cast(embedding_table, self.dtype))
        return logits


class PanguAlphaModel(nn.Cell):
    """
    The GPT network consisting of two parts the backbone and the head
    Args:
        config(GPTConfig): the config of network
    Inputs:
        input_ids: the tokenized inputs
        input_mask: the mask indicating whether each position is a valid input
        past: the previous feature map
    Returns:
        logits: Tensor: the logits of the corresponding inputs with shape (batch_size, seq_length, vocab_size)
    """

    def __init__(self, config):
        super(PanguAlphaModel, self).__init__()
        self.backbone = PanguAlpha_Model(config)
        self.head = PanGuHead(config)

    def construct(self, input_ids, input_mask, init_reset=True, batch_valid_length=None, input_position=None,
                  attention_mask=None, past=None):
        output_states, _, embedding_table = self.backbone(
            input_ids, input_mask, init_reset, batch_valid_length, input_position, attention_mask, past)
        logits = self.head(output_states, embedding_table)
        return logits


class CrossEntropyLoss(nn.Cell):
    """
    Calculate the cross entropy loss
    Args:
        config(GPTConfig): the config of the network
    Inputs:
        logits: the output logits of the backbone
        label: the ground truth label of the sample
        input_mask: the mask indicating whether each position is a valid input
    Returns:
        loss: Tensor, the corrsponding cross entropy loss
    """

    def __init__(self, config):
        super(CrossEntropyLoss, self).__init__()
        self.mean = P.ReduceMean()
        self.sum = P.ReduceSum().shard(((config.dp, config.mp),))
        self.onehot = P.OneHot().shard(((config.dp, config.mp), (), ()))
        self.on_value = Tensor(1.0, mstype.float32)
        self.off_value = Tensor(0.0, mstype.float32)
        self.vocab_size = config.vocab_size
        self.max = P.ArgMaxWithValue(axis=-1, keep_dims=True).shard(
            ((config.dp, config.mp),))
        self.eps_const = Tensor(1e-24, mstype.float32)
        self.sub = P.Sub().shard(((config.dp, config.mp), (config.dp, 1)))
        self.exp = P.Exp().shard(((config.dp, config.mp),))
        self.div = P.RealDiv().shard(((config.dp, config.mp), (config.dp, 1)))
        self.log = P.Log().shard(((config.dp, config.mp),))
        self.add = P.TensorAdd().shard(((config.dp, config.mp), ()))
        self.mul = P.Mul().shard(
            ((config.dp, config.mp), (config.dp, config.mp)))
        self.neg = P.Neg().shard(((config.dp, config.mp),))
        self.sum2 = P.ReduceSum().shard(((1,),))

        self.mul2 = P.Mul().shard(((1,), (1,)))
        self.add2 = P.TensorAdd()
        self.div2 = P.RealDiv()

    def construct(self, logits, label, input_mask):
        logits = F.cast(logits, mstype.float32)
        _, logit_max = self.max(logits)
        logit_sub = self.sub(logits, logit_max)
        logit_exp = self.exp(logit_sub)
        exp_sum = self.sum(logit_exp, -1)
        exp_sum = P.Reshape()(exp_sum, (F.shape(exp_sum)[0], 1))
        softmax_result = self.div(logit_exp, exp_sum)
        log_softmax_result = self.log(self.add(softmax_result, self.eps_const))
        label = P.Reshape()(label, (-1,))
        one_hot_label = self.onehot(label, self.vocab_size, self.on_value,
                                    self.off_value)
        loss = self.mul(log_softmax_result, one_hot_label)
        loss_unsum = self.neg(loss)
        loss_reduce = self.sum(loss_unsum, -1)
        input_mask = P.Reshape()(input_mask, (-1,))
        numerator = self.sum2(self.mul2(loss_reduce, input_mask))

        denominator = self.add2(
            self.sum2(input_mask),
            P.Cast()(F.tuple_to_array((1e-5,)), mstype.float32))
        loss = self.div2(numerator, denominator)
        return loss


class PanGuAlphaWithLoss(nn.Cell):
    """
    GPT training loss
    Args:
        network: backbone network of GPT2/3
        loss: loss function, e.g., crossentropy
        eos_token: the end_of_sentence token
    Inputs:
        input_ids: the tokenized inputs
        past: the previous feature map
    Returns:
        output: Tensor, the loss of the network
    """

    def __init__(self, config, network, loss, eos_token=6):
        super(PanGuAlphaWithLoss, self).__init__(auto_prefix=False)
        self.network = network
        self.loss = loss
        self.eos_token = eos_token
        self.slice = P.StridedSlice().shard(((config.dp, 1),))
        self.not_equal = P.NotEqual().shard(((config.dp, 1), ()))
        self.batch_size = config.batch_size
        self.len = config.seq_length
        self.eod_reset = config.eod_reset
        if self.eod_reset:
            self.slice_mask = P.StridedSlice().shard(((config.dp, 1, 1),))

    def construct(self, input_ids, input_position=None, attention_mask=None):
        # tokens = input_ids[:, :-1]
        tokens = self.slice(input_ids, (0, 0), (self.batch_size, -1), (1, 1))

        if self.eod_reset:
            input_position = self.slice(input_position, (0, 0), (self.batch_size, self.len), (1, 1))
            attention_mask = self.slice_mask(attention_mask, (0, 0, 0),
                                             (self.batch_size, self.len, self.len),
                                             (1, 1, 1))

        input_mask = F.cast(self.not_equal(tokens, self.eos_token),
                            mstype.float32)
        logits = self.network(tokens, input_mask, input_position, attention_mask)
        # labels = input_ids[:, 1:]
        labels = self.slice(input_ids, (0, 1), (self.batch_size, self.len + 1),
                            (1, 1))
        output = self.loss(logits, labels, input_mask)
        return output


class EvalNet(nn.Cell):
    """
    GPT evaluation net
    Args:
        backbone: backbone network of GPT2/3
        generate: enable generate mode
        topk_num: the number of topk selection
    Inputs:
        input_ids: the tokenized inpus
        init_reset: whether to reset key_past and value_past for incremental inference
        batch_valid_length: the original valid sentence length of the first graph for incremental inference
    Returns:
        outputs: Tensor, corresponding output for different tasks
    """

    def __init__(self, backbone, generate=False, topk_num=3, pad_token=9):
        super(EvalNet, self).__init__(auto_prefix=False)
        self.backbone = backbone
        self.argmax = P.Argmax()
        self.generate = generate
        self.topk = P.TopK(sorted=True).shard(((1, 1, 1),))
        self.not_equal = P.NotEqual().shard(((1, 1), ()))
        self.topk_num = topk_num
        self.pad_token = pad_token  # config.pad_token

    def construct(self, input_ids, init_reset=True, batch_valid_length=None):
        """
            evaluation net
            input_ids: bs x seq_length
        """
        input_mask = F.cast(self.not_equal(input_ids, self.pad_token), mstype.float32)
        logits = self.backbone(input_ids, input_mask, init_reset, batch_valid_length)
        bs, seq_length = F.shape(input_ids)
        logits = F.reshape(logits, (bs, seq_length, -1))
        value, index = self.topk(logits, self.topk_num)
        return value, index


class EvalNet_p(nn.Cell):
    """
    GPT evaluation net
    Args:
        backbone: backbone network of GPT2/3
        generate: enable generate mode
        topk_num: the number of topk selection
    Inputs:
        input_ids: the tokenized inpus
        init_reset: whether to reset key_past and value_past for incremental inference
        batch_valid_length: the original valid sentence length of the first graph for incremental inference
    Returns:
        outputs: Tensor, corresponding output for different tasks
    """

    def __init__(self, backbone, generate=False, pad_token=9, topk_num=3):
        super(EvalNet_p, self).__init__(auto_prefix=False)
        self.backbone = backbone
        self.argmax = P.Argmax()
        self.generate = generate
        self.not_equal = P.NotEqual().shard(((1, 1), ()))
        self.log_max = P.LogSoftmax().shard(((1, 1, 1),))
        self.topk_num = topk_num
        self.pad_token = pad_token  # config.pad_token

    def construct(self, input_ids, init_reset=True, batch_valid_length=None):
        """
            evaluation net
            input_ids: bs x seq_length
        """
        input_mask = F.cast(self.not_equal(input_ids, self.pad_token), mstype.float32)
        logits = self.backbone(input_ids, input_mask, init_reset, batch_valid_length)
        bs, seq_length = F.shape(input_ids)

        logits = F.reshape(logits, (bs, seq_length, -1))
        if self.generate:
            outputs = self.log_max(logits)
            outputs = F.tensor_pow(np.e, outputs)
        else:
            outputs = self.argmax(logits)
        return outputs


class EvalNet(nn.Cell):
    """
    PanguAlpha evaluation net
    Args:
        backbone: backbone network of PanguAlpha
        generate: enable generate mode
    Inputs:
        input_ids: the tokenized inpus
        current_index: the index of current token
        init_reset: whether reset saved states
    Returns:
        outputs: Tensor, corresponding output for different tasks
    """

    def __init__(self, backbone, generate=False, pad_token=6, seq_length=1024):
        super(EvalNet, self).__init__(auto_prefix=False)
        self.backbone = backbone
        self.pad_token = pad_token
        self.argmax = P.Argmax()
        self.generate = generate
        self.topk = P.TopK(sorted=True).shard(((1, 1),))
        self.gather = P.GatherV2().shard(((1, 1), (1,)))
        self.log_softmax = P.LogSoftmax().shard(((1, 1, 1),))
        self.get_attention_mask = AttentionMask(seq_length)

    def construct(self, input_ids, current_index, init_reset=True, batch_valid_length=None):
        """evaluation net"""
        input_mask = F.cast(F.not_equal(input_ids, self.pad_token), mstype.float32)
        bs, seq_length = F.shape(input_ids)
        attention_mask = self.get_attention_mask(input_mask)
        input_position = F.tuple_to_array(F.make_range(seq_length))
        input_position = P.Tile()(input_position, (bs, 1))
        logits = self.backbone(input_ids, input_mask, input_position, attention_mask,
                               init_reset, batch_valid_length)
        index = current_index.view(1, )
        logits = self.gather(logits, index, 0)
        logits = logits.view(bs, 1, -1)
        log_probs = self.log_softmax(logits)
        return log_probs